<?php

namespace Drupal\recaptcha_preloader\Service;

use Drupal\Core\Asset\AssetResolverInterface;
use Drupal\Core\Asset\AttachedAssets;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Template\Attribute;

/**
 * Class RecaptchaPreloaderHelper.
 *
 * @package Drupal\recaptcha_preloader\Service
 */
class RecaptchaPreloaderHelper implements RecaptchaPreloaderHelperInterface {

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The asset resolver.
   *
   * @var \Drupal\Core\Asset\AssetResolverInterface
   */
  protected $assetResolver;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * RecaptchaPreloaderHelper constructor.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \Drupal\Core\Asset\AssetResolverInterface $asset_resolver
   *   The asset resolver.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer.
   */
  public function __construct(
    ModuleHandlerInterface $module_handler,
    ConfigFactoryInterface $config_factory,
    AssetResolverInterface $asset_resolver,
    LanguageManagerInterface $language_manager,
    RendererInterface $renderer
  ) {
    $this->moduleHandler = $module_handler;
    $this->configFactory = $config_factory;
    $this->assetResolver = $asset_resolver;
    $this->languageManager = $language_manager;
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public function search(array &$elements) {
    foreach (Element::children($elements) as $key) {
      if (is_array($elements[$key])) {
        switch ($elements[$key]['#type']) {
          case 'actions':
            $this->search($elements[$key]);
            break;

          case 'submit':
            $elements[$key]['#disabled'] = TRUE;
            $elements[$key]['#attributes']['class'][] = 'blocked-by-recaptcha';
            break;
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $captcha = $this->moduleHandler->invoke(
      'recaptcha',
      'captcha',
      ['generate', self::CHALLENGE_TYPE]
    );

    if (!isset($captcha['form']['recaptcha_widget'])) {
      return $captcha;
    }

    $recaptcha_config = $this->configFactory->get('recaptcha.settings');

    if ($recaptcha_config->get('widget.noscript')) {
      return $captcha;
    }

    // Allow using only with default options set of the widget size setting.
    if (($size = $recaptcha_config->get('widget.size')) && $size !== 'compact') {
      return $captcha;
    }

    $config = $this->configFactory->get('recaptcha_preloader.settings');

    if (!$config->get('use')) {
      return $captcha;
    }

    $assets = AttachedAssets::createFromRenderArray([
      '#attached' => [
        'library' => [
          'recaptcha_preloader/connector',
        ],
      ],
    ]);

    $assets = [
      'css' => [$this->assetResolver->getCssAssets($assets, FALSE)],
      'js' => $this->assetResolver->getJsAssets($assets, FALSE),
    ];

    $fields = [
      'css' => [
        'tag' => 'link',
        'attribute' => 'href',
        'attributes' => [
          'rel' => 'stylesheet',
          'type' => 'text/css',
        ],
      ],
      'js' => [
        'tag' => 'script',
        'attribute' => 'src',
        'attributes' => [
          'async' => TRUE,
          'defer' => TRUE,
        ],
      ],
    ];

    $widget = &$captcha['form']['recaptcha_widget'];
    $widget['#attached']['html_head'] = [];
    $langcode = $this->languageManager->getCurrentLanguage()->getId();

    foreach ($assets as $type => $group) {
      $id = 1;

      foreach ($group as $items) {
        foreach ($items as $item) {
          $url = $item['data'];

          if ($item['type'] === 'external') {
            $url .= '&hl=' . $langcode;
          }
          else {
            $url = '/' . $url;
          }

          $widget['#attached']['html_head'][] = [
            [
              '#tag' => $fields[$type]['tag'],
              '#attributes' => [
                  $fields[$type]['attribute'] => $url,
                ] + $fields[$type]['attributes'],
            ],
            'recaptcha_preloader_' . $type . '_' . ($id++),
          ];
        }
      }
    }

    $attributes = [
      'class' => [
        'g-recaptcha-wrapper',
        $size ?: 'default',
        'loading',
      ],
    ];

    $widget['#prefix'] = '<div' . new Attribute($attributes) . '>';

    if ($message = $config->get('message')) {
      if ($config->get('appearance')) {
        $element = [
          '#type' => 'html_tag',
          '#tag' => 'div',
          '#attributes' => [
            'class' => [
              'g-recaptcha-message',
              $recaptcha_config->get('widget.theme') ?: 'light',
            ],
          ],
          '#value' => $message,
        ];

        $widget['#suffix'] .= $this->renderer->render($element);
      }
      else {
        $widget['#markup'] = substr($widget['#markup'], 0, -6) . $message . substr($widget['#markup'], -6);
      }
    }

    $widget['#suffix'] .= '</div>';

    return $captcha;
  }

}
